/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_ftp.h
 *
 * Purpose:     Win32-specific header file for the recls API.
 *
 * Created:     18th August 2003
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLS_FTP
#define RECLS_INCL_H_RECLS_FTP

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLS_FTP_MAJOR    2
# define RECLS_VER_H_RECLS_FTP_MINOR    0
# define RECLS_VER_H_RECLS_FTP_REVISION 2
# define RECLS_VER_H_RECLS_FTP_EDIT     4
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_ftp.h FTP-specific parts of the \ref group_recls  API */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

#if defined(RECLS_API_FTP)
# undef RECLS_API_FTP
#endif /* RECLS_API_FTP */

#if !defined(RECLS_NO_API_FTP)
# ifndef RECLS_PLATFORM_IS_WIN32
#  error The recls-FTP functionality is currently available only in Win32 compilations
# endif /* RECLS_PLATFORM_IS_WIN32 */

# define RECLS_API_FTP


/***************************************
 * Search control
 */

/** \name Search control functions
 *
 * \ingroup group_recls
 */
/** @{ */




/** Searches a given directory for matching files of the given pattern
 *
 *
 * \param host The host on which to search
 * \param username The username with which to gain access
 * \param password The password with which to gain access
 * \param searchRoot The directory representing the root of the search
 * \param pattern The search pattern, e.g. "*.c"
 * \param flags A combination of 0 or more RECLS_FLAG values
 * \param phSrch Address of the search handle. This is set to NULL on failure
 * \return A status code indicating success/failure
 *
 */
RECLS_FNDECL(recls_rc_t) Recls_SearchFtp(       recls_char_t const          *host
                                    ,           recls_char_t const          *username
                                    ,           recls_char_t const          *password
                                    ,           recls_char_t const          *searchRoot
                                    ,           recls_char_t const          *pattern
                                    ,           recls_uint32_t              flags
                                    ,           hrecls_t                    *phSrch);

/** @} */

#endif /* !RECLS_NO_API_FTP */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLS_FTP */

/* ////////////////////////////////////////////////////////////////////////// */
