/* /////////////////////////////////////////////////////////////////////////////
 * File:        libreclsch.c
 *
 * Purpose:     Implementation file for the recls - Ch mapping
 *
 * Created:     9th August 2004
 * Updated:     10th January 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* \file libreclsch.c Implementation file for the recls - Ch mapping
 *
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <ch.h>
#include <recls.h>
#include <recls_assert.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#define IMPLEMENT_0_void(F)				\
										\
	EXPORTCH void F##_chdl(void *arg_)	\
	{									\
		((void)arg_);					\
										\
		F();							\
	}

#define	IMPLEMENT_0_R(R, F)				\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
										\
		((void)arg_);					\
										\
		retVal = F();					\
										\
		return retVal;					\
	}

#define IMPLEMENT_1_void(F, T0)			\
										\
	EXPORTCH void F##_chdl(void *arg_)	\
	{									\
		va_list		ap;					\
		T0			arg0_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
										\
		Ch_VaEnd(ap);					\
										\
		F(arg0_);						\
	}

#define	IMPLEMENT_1_R(R, F, T0)			\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
		T0			arg0_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
										\
		Ch_VaEnd(ap);					\
										\
		retVal = F(arg0_);				\
										\
		return retVal;					\
	}

#define IMPLEMENT_2_void(F, T0, T1)		\
										\
	EXPORTCH void F##_chdl(void *arg_)	\
	{									\
		va_list		ap;					\
		T0			arg0_;				\
		T1			arg1_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
										\
		Ch_VaEnd(ap);					\
										\
		F(arg0_, arg1_);				\
	}

#define	IMPLEMENT_2_R(R, F, T0, T1)		\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
		T0			arg0_;				\
		T1			arg1_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
										\
		Ch_VaEnd(ap);					\
										\
		retVal = F(arg0_, arg1_);		\
										\
		return retVal;					\
	}

#define IMPLEMENT_3_void(F, T0, T1, T2)	\
										\
	EXPORTCH void F##_chdl(void *arg_)	\
	{									\
		va_list		ap;					\
		T0			arg0_;				\
		T1			arg1_;				\
		T2			arg2_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
		arg2_	=	Ch_VaArg(ap, T2);	\
										\
		Ch_VaEnd(ap);					\
										\
		F(arg0_, arg1_, arg2_);			\
	}

#define	IMPLEMENT_3_R(R, F, T0, T1, T2)	\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
		T0			arg0_;				\
		T1			arg1_;				\
		T2			arg2_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
		arg2_	=	Ch_VaArg(ap, T2);	\
										\
		Ch_VaEnd(ap);					\
										\
		retVal = F(arg0_, arg1_, arg2_);	\
										\
		return retVal;					\
	}



#define	IMPLEMENT_4_R(R, F, T0, T1, T2, T3)	\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
		T0			arg0_;				\
		T1			arg1_;				\
		T2			arg2_;				\
		T3			arg3_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
		arg2_	=	Ch_VaArg(ap, T2);	\
		arg3_	=	Ch_VaArg(ap, T3);	\
										\
		Ch_VaEnd(ap);					\
										\
		retVal = F(arg0_, arg1_, arg2_, arg3_);	\
										\
		return retVal;					\
	}


#define	IMPLEMENT_5_R(R, F, T0, T1, T2, T3, T4)	\
										\
	EXPORTCH R F##_chdl(void *arg_)		\
	{									\
		va_list		ap;					\
		R			retVal;				\
		T0			arg0_;				\
		T1			arg1_;				\
		T2			arg2_;				\
		T3			arg3_;				\
		T4			arg4_;				\
										\
		Ch_VaStart(ap, arg_);			\
										\
		arg0_	=	Ch_VaArg(ap, T0);	\
		arg1_	=	Ch_VaArg(ap, T1);	\
		arg2_	=	Ch_VaArg(ap, T2);	\
		arg3_	=	Ch_VaArg(ap, T3);	\
		arg4_	=	Ch_VaArg(ap, T4);	\
										\
		Ch_VaEnd(ap);					\
										\
		retVal = F(arg0_, arg1_, arg2_, arg3_, arg4_);	\
										\
		return retVal;					\
	}

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/***************************************
 * File system
 */

IMPLEMENT_0_R(recls_char_t const*, Recls_GetPathNameSeparator)
IMPLEMENT_0_R(recls_char_t const*, Recls_GetPathSeparator)
IMPLEMENT_0_R(recls_char_t const*, Recls_GetWildcardsAll)

/***************************************
 * Search control
 */

IMPLEMENT_4_R(recls_rc_t, Recls_Search, char const*, char const*, unsigned, hrecls_t*)
IMPLEMENT_5_R(recls_rc_t, Recls_SearchProcess, recls_char_t const*, recls_char_t const*, recls_uint32_t, hrecls_process_fn_t, recls_process_fn_param_t)
IMPLEMENT_1_void(Recls_SearchClose, hrecls_t)

/***************************************
 * Search enumeration
 */

IMPLEMENT_1_R(recls_rc_t, Recls_GetNext, hrecls_t)
IMPLEMENT_2_R(recls_rc_t, Recls_GetDetails, hrecls_t, recls_info_t*)
IMPLEMENT_2_R(recls_rc_t, Recls_GetNextDetails, hrecls_t, recls_info_t*)

/***************************************
 * File entry information
 */

IMPLEMENT_1_void(Recls_CloseDetails, recls_info_t)
IMPLEMENT_2_R(recls_rc_t, Recls_CopyDetails, recls_info_t, recls_info_t*)
IMPLEMENT_2_R(recls_rc_t, Recls_OutstandingDetails, hrecls_t, recls_uint32_t*)

/***************************************
 * Error handling
 */

IMPLEMENT_1_R(recls_rc_t, Recls_GetLastError, hrecls_t)
IMPLEMENT_3_R(size_t, Recls_GetErrorString, recls_rc_t, char*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetLastErrorString, hrecls_t, recls_char_t*, size_t)

/***************************************
 * Property elicitation
 */

IMPLEMENT_3_R(size_t, Recls_GetPathProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetDirectoryProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetDirectoryPathProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetFileProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetShortFileProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetFileNameProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_3_R(size_t, Recls_GetFileExtProperty, recls_info_t, recls_char_t*, size_t)
IMPLEMENT_4_R(size_t, Recls_GetDirectoryPartProperty, recls_info_t, int, recls_char_t*, size_t)
IMPLEMENT_1_R(recls_bool_t, Recls_IsFileReadOnly, recls_info_t)
IMPLEMENT_1_R(recls_bool_t, Recls_IsFileDirectory, recls_info_t)
IMPLEMENT_1_R(recls_bool_t, Recls_IsFileLink, recls_info_t)
IMPLEMENT_2_void(Recls_GetSizeProperty, recls_info_t, recls_filesize_t*)
IMPLEMENT_1_R(recls_time_t, Recls_GetCreationTime, recls_info_t)
IMPLEMENT_1_R(recls_time_t, Recls_GetModificationTime, recls_info_t)
IMPLEMENT_1_R(recls_time_t, Recls_GetLastAccessTime, recls_info_t)
IMPLEMENT_1_R(recls_time_t, Recls_GetLastStatusChangeTime, recls_info_t)

/***************************************
 * Extended API functions
 */

IMPLEMENT_1_R(recls_bool_t, Recls_IsDirectoryEmpty, recls_char_t const*)
IMPLEMENT_1_R(recls_bool_t, Recls_IsDirectoryEntryEmpty, recls_info_t)
IMPLEMENT_1_R(recls_filesize_t, Recls_CalcDirectorySize, recls_char_t const*)
IMPLEMENT_1_R(recls_filesize_t, Recls_CalcDirectoryEntrySize, recls_info_t)

/* ////////////////////////////////////////////////////////////////////////// */
