/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_jni_util.h
 *
 * Purpose:     Utility file for Java mapping
 *
 * Created      21st November 2003
 * Updated:     20th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <recls.h>

#include <jni.h>

#include <stlsoft.h>
#if !defined(_STLSOFT_VER_1_8_3)
# error STLSoft version 1.8.3 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#include <time.h>

/* ////////////////////////////////////////////////////////////////////////// */

#if !defined(RECLS_NO_NAMESPACE)
using recls::recls_rc_t;
using recls::recls_bool_t;
using recls::recls_char_t;
using recls::recls_time_t;
using recls::recls_filesize_t;
using recls::recls_root_t;

using recls::RECLS_FAILED;
using recls::RECLS_SUCCEEDED;
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(__cplusplus)
static const char	CLASSNAME_ENTRY[]			=	"Lorg/recls/Entry;";
static const char	CLASSNAME_STRING[]			=	"Ljava/lang/String;";
static const char	CLASSNAME_SEARCH[]			=	"Lorg/recls/Search;";
static const char	CLASSNAME_RECLSX[]			=	"Lorg/recls/ReclsException;";
static const char	CLASSNAME_UNSUPPORTEDX[]	=	"Ljava/lang/UnsupportedOperationException;";
static const char	CLASSNAME_ILLEGALSTATEX[]	=	"Ljava/lang/IllegalStateException;";
static const char	CLASSNAME_NOSUCHELEMENTX[]	=	"Ljava/lang/NoSuchElementException;";
static const char	FIELDNAME_HENTRY[]			=	"m_hEntry";
static const char	FIELDNAME_HSRCH[]			=	"m_hSearch";
#else /* ? __cplusplus */
# define			CLASSNAME_ENTRY					"Lorg/recls/Entry;"
# define			CLASSNAME_STRING				"Ljava/lang/String;"
# define			CLASSNAME_SEARCH				"Lorg/recls/Search;"
# define			CLASSNAME_RECLSX				"Lorg/recls/ReclsException;"
# define			CLASSNAME_UNSUPPORTEDX			"Ljava/lang/UnsupportedOperationException;"
# define			CLASSNAME_ILLEGALSTATEX			"Ljava/lang/IllegalStateException;"
# define			CLASSNAME_NOSUCHELEMENTX		"Ljava/lang/NoSuchElementException;"
# define			FIELDNAME_HENTRY				"m_hEntry"
# define			FIELDNAME_HSRCH					"m_hSearch"
#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

void ThrowException(JNIEnv *henv, char const *className, char const *message);
void ThrowReclsException(JNIEnv *henv, char const *message, recls_rc_t code);


jstring StringFromRange(JNIEnv *henv, recls_char_t const *from, recls_char_t const *to);

jsize SafeStringUTFLength(JNIEnv *henv, jstring str);
jsize SafeStringLength(JNIEnv *henv, jstring str);

char const *SafeStringUTFChars(JNIEnv *henv, jstring str, jboolean *isCopy);
jchar const *SafeStringChars(JNIEnv *henv, jstring str, jboolean *isCopy);

/* ////////////////////////////////////////////////////////////////////////// */
