/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFtpSearchDirectoryNode_win32.h
 *
 * Purpose:     ReclsFtpSearchDirectoryNode class, for Win32.
 *
 * Created:     31st May 2004
 * Updated:     20th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file ReclsFtpSearchDirectoryNode_win32.h Definition of the ReclsFtpSearchDirectoryNode class
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

// recls includes
#include <recls.h>
#include "recls_string.h"
#include "recls_util.h"

// STLSoft includes
#include <stlsoft.h>
#if !defined(_STLSOFT_VER_1_8_3)
# error STLSoft version 1.8.3 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

// Platform-specific includes
#include <inetstl.h>
#include <inetstl/filesystem_traits.hpp>

#ifdef RECLS_USE_NEW_INETSTL_FFS
# include <inetstl/findfile_sequence.hpp>
#else /* ? RECLS_USE_NEW_INETSTL_FFS */
# include <inetstl/findfile_sequence.hpp>
# ifdef _RECLS_SUPPORTS_MULTIPATTERN
#  include <inetstl/searchspec_sequence.hpp>
# endif /* _RECLS_SUPPORTS_MULTIPATTERN */
#endif /* RECLS_USE_NEW_INETSTL_FFS */


// The Dinkumware list implementation causes Visual C++ 7.1 to give C4702 warning, so we don't use it
#if defined(RECLS_COMPILER_IS_MSVC) && \
    _MSC_VER >= 1310
# include <vector>
#else /* ? compiler */
# include <list>
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

inetstl_ns_using(basic_findfile_sequence)

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

class ReclsFtpSearchDirectoryNode
    : public ReclsSearchDirectoryNode
{
public:
    typedef ReclsFtpSearchDirectoryNode                                     class_type;
    typedef inetstl_ns_qual(filesystem_traits)<recls_char_t>                traits_type;
private:
    typedef RECLS_STRING_TEMPLATE_1(recls_char_t)                           string_type;
    typedef basic_findfile_sequence<recls_char_t, traits_type>              file_find_sequence_type;
#if defined(RECLS_COMPILER_IS_MSVC) && \
    _MSC_VER >= 1310
    typedef stlsoft_ns_qual_std(vector)<string_type>                        directory_sequence_type;
#else /* ? compiler */
    typedef stlsoft_ns_qual_std(list)<string_type>                          directory_sequence_type;
#endif /* compiler */
#ifdef RECLS_USE_NEW_INETSTL_FFS
    typedef file_find_sequence_type                                         entry_sequence_type;
#else /* ? RECLS_USE_NEW_INETSTL_FFS */
# ifdef _RECLS_SUPPORTS_MULTIPATTERN
    typedef inetstl_ns_qual(searchspec_sequence)<file_find_sequence_type>   entry_sequence_type;
# else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
    typedef file_find_sequence_type                                         entry_sequence_type;
# endif /* _RECLS_SUPPORTS_MULTIPATTERN */
#endif /* RECLS_USE_NEW_INETSTL_FFS */

// Construction
protected: // Not private, or GCC whines
    ReclsFtpSearchDirectoryNode(HINTERNET connection, recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *pattern);
public:
    virtual ~ReclsFtpSearchDirectoryNode();

    static class_type *FindAndCreate(HINTERNET connection, recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *pattern);
    static class_type *FindAndCreate(HINTERNET connection, recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *subDir, recls_char_t const *pattern);

// ReclsSearchDirectoryNode methods
public:
    /* virtual */ recls_rc_t GetNext();
    /* virtual */ recls_rc_t GetDetails(recls_info_t *pinfo);
    /* virtual */ recls_rc_t GetNextDetails(recls_info_t *pinfo);

// Implementation
private:
    recls_rc_t      Initialise();

#ifdef RECLS_ENFORCING_CONTRACTS
    recls_bool_t    is_valid() const;
#endif /* RECLS_ENFORCING_CONTRACTS */

#if defined(RECLS_COMPILER_IS_BORLAND)
    static directory_sequence_type::const_iterator _select_iter(int b, directory_sequence_type::const_iterator trueVal, directory_sequence_type::const_iterator falseVal)
    {
        // I can't explain it, but Borland does not like the tertiary operator and the copy-ctors of the iterators
        if(b)
        {
            return trueVal;
        }
        else
        {
            return falseVal;
        }
    }
#endif /* !RECLS_COMPILER_IS_BORLAND */
    static int _ssFlags_from_reclsFlags(recls_uint32_t flags)
    {
        int ssFlags = 0;

        if(0 != (flags & RECLS_F_FILES))
        {
#if defined(_RECLS_SUPPORTS_MULTIPATTERN) && \
    !defined(RECLS_USE_NEW_INETSTL_FFS)
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::files;

//            ssFlags |= entry_sequence_type::find_sequence_type::files;
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
            ssFlags |= entry_sequence_type::files;
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */
        }
        if(0 != (flags & RECLS_F_DIRECTORIES))
        {
#if defined(_RECLS_SUPPORTS_MULTIPATTERN) && \
    !defined(RECLS_USE_NEW_INETSTL_FFS)
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::directories;

//            ssFlags |= entry_sequence_type::find_sequence_type::directories;
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
            ssFlags |= entry_sequence_type::directories;
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */
        }

        return ssFlags;
    }

// Implementation
private:
    static directory_sequence_type init_directories_(HINTERNET connection, recls_char_t const *rootDir, recls_uint32_t flags);

// Members
private:
    HINTERNET                               m_connection;
    recls_info_t                            m_current;
    class_type                              *m_dnode;
    recls_uint32_t const                    m_flags;
    string_type                             m_rootDir;
    string_type                             m_pattern;
    directory_sequence_type                 m_directories;
    directory_sequence_type::const_iterator m_directoriesBegin;
    entry_sequence_type                     m_entries;
    entry_sequence_type::const_iterator     m_entriesBegin;

// Not to be implemented
private:
    ReclsFtpSearchDirectoryNode(class_type const &);
    class_type &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
