/* /////////////////////////////////////////////////////////////////////////////
 * File:        STL.cpp
 *
 * Purpose:     Implementation file for the STL project.
 *
 * Created:     16th August 2003
 * Updated:     23rd May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2003-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

// Define this definition to retrict use to platform-independent aspects of the API
//#define RECLS_PURE_API

// Define this to copy file entries into a vector, to exercise the copy semantics
//#define TAKE_COPIES_IN_VECTOR

#include <recls.h>
#ifdef RECLS_PLATFORM_IS_WIN32
# include <recls_ftp.h>
#endif /* RECLS_PLATFORM_IS_WIN32 */

#if RECLS_VER < RECLS_VER_1_6_1
# error This file now requires version 1.6.1 or later of the recls C API
#endif /* RECLS_VER < 1.6.1 */

#ifdef RECLS_STRICT
# include <stlsoft_nulldef.h>
#endif // RECLS_STRICT

#include <stlsoft.h>
#if defined(STLSOFT_COMPILER_IS_INTEL) || \
	defined(STLSOFT_COMPILER_IS_MSVC)
# include <crtdbg.h>
# pragma warning(disable : 4530)
#endif /* compiler */

#include <recls/stl/recls.hpp>
#include <recls/stl/root_sequence.hpp>
#include <recls/stl/search_sequence.hpp>

#if !defined(RECLS_COMPILER_IS_DMC)
# include <iostream>
# include <iomanip>
#else /* ? compiler */
# include <iostream.h>
# include <iomanip.h>
#endif /* compiler */

#include <stdio.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_COMPILER_IS_DMC)
using std::cout;
using std::cerr;
using std::endl;
using std::setw;
#endif /* compiler */
using recls::recls_info_t;
using recls::recls_char_t;
using recls::RECLS_F_FILES;
using recls::RECLS_F_DIRECTORIES;
using recls::RECLS_F_RECURSIVE;
using recls::RECLS_F_DIRECTORY_PARTS;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
int processDirectory(char const *host, char const *username, char const *password, char const *rootDir, char const *pattern, long flags, int bSuccinct);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState    memState;
#endif /* _MSC_VER && _MSC_VER */
    int             iRet            =   0;
    int             i;
    int             totalFound      =   0;
    char const      *host	        =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern        =   NULL;
    char const      *rootDir        =   NULL;
    unsigned        flags           =   RECLS_F_RECURSIVE;
    int             bSuccinct       =   false;
    int             bAllHardDrives  =   false;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = 1;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = true;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

	if( NULL != password &&
		(	NULL == host ||
			NULL == username))
	{
		usage(1, "Must specify host and username if specifying password");
	}

	if( NULL != username &&
		NULL == host)
	{
		usage(1, "Must specify host if specifying username");
	}

	if( NULL != host &&
		bAllHardDrives)
	{
		usage(1, "-h flag meaningless to FTP searches");
	}

    if( NULL != host &&
		NULL == rootDir)
    {
		usage(1, "Must specify a root directory for FTP searches");
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

    // Search for files if neither files or directories specified
    //
    // Even though this is not necessary, because the recls API provides the
    // same interpretation, it's best to be explicit.
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

    if(NULL == pattern)
    {
        pattern = recls::Recls_GetWildcardsAll();
    }

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

    if(bAllHardDrives)
    {
        reclstl::basic_root_sequence<recls_char_t>                  roots;
        reclstl::basic_root_sequence<recls_char_t>::const_iterator  begin   =   roots.begin();
        reclstl::basic_root_sequence<recls_char_t>::const_iterator  end     =   roots.end();

        for(; begin != end; ++begin)
        {
            totalFound += processDirectory(host, username, password, *begin, pattern, flags, bSuccinct);
        }
    }
    else
    {
        totalFound = processDirectory(host, username, password, rootDir, pattern, flags, bSuccinct);
    }

    fprintf(stdout, "Total found: %d\n", totalFound);

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit, char const *reason)
{
    cerr << "recls STL Test Program:" << endl << endl;

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n\n", reason);
	}

    cerr << "Usage: STL [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]" << endl;
    cerr << "\t-R               -   does not recurse; recursive search is the default" << endl;
    cerr << "\t-p               -   evaluate and display directory parts" << endl;
    cerr << "\t-d               -   search for directories" << endl;
    cerr << "\t-f               -   search for files (this is the default, if -d is not specified" << endl;
    cerr << "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)" << endl;
    cerr << "\t-s               -   succinct; shows only the path" << endl;
	cerr << "\t-H<host>         -   performs an FTP search on the given host" << endl;
	cerr << "\t-U<username>     -   username to use to log on to <host>" << endl;
	cerr << "\t-P<password>     -   password to use to log on to <host>" << endl;
    cerr << "\t<pattern>        -   search pattern, e.g. \"*.cpp\"; default is to search for all files" << endl;
    cerr << "\t<root-dir>       -   root directory of search; default is current working directory" << endl;

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}


int processDirectory(char const *host, char const *username, char const *password, char const *rootDir, char const *pattern, long flags, int bSuccinct)
{
    typedef reclstl::basic_search_sequence<recls_char_t>    sequence_t;

    int         totalFound      =   0;
#if defined(RECLS_API_FTP)
    sequence_t  search(host, username, password, rootDir, pattern, flags);
#else /* ? RECLS_API_FTP */
	STLSOFT_SUPPRESS_UNUSED(host);
	STLSOFT_SUPPRESS_UNUSED(username);
	STLSOFT_SUPPRESS_UNUSED(password);
    sequence_t  search(rootDir, pattern, flags);
#endif /* RECLS_API_FTP */

    sequence_t::const_iterator    begin   =   search.begin();
    sequence_t::const_iterator    end     =   search.end();

    for(; begin != end; ++begin)
    {
        using reclstl::string_t;

        ++totalFound;

        sequence_t::value_type    fileEntry   =   *begin;

#ifdef RECLS_PLATFORM_API_WIN32
        recls_char_t            drive       =   fileEntry.get_drive();
#endif /* RECLS_PLATFORM_API_WIN32 */
        string_t                uncDrive    =   fileEntry.get_UNC_drive();
        string_t                path        =   fileEntry.get_path();
        string_t                dir         =   fileEntry.get_directory();
        string_t                dirPath     =   fileEntry.get_directory_path();
        string_t                file        =   fileEntry.get_file();
        string_t                fileName    =   fileEntry.get_filename();
        string_t                fileExt     =   fileEntry.get_fileext();

        string_t                pathCheck;
        string_t                dirCheck;
        string_t                fileCheck;

        cout << "  " << path << endl;
        if(!bSuccinct)
        {
            int off = 0;

            cout << "  " << dirPath << endl;
			if(fileEntry.is_UNC())
			{
	            cout << "  " << uncDrive << endl;
				pathCheck += uncDrive;

				off += uncDrive.length();
			}
			else
			{
#ifdef RECLS_PLATFORM_API_WIN32
				if(isupper(path[0]))
				{
					drive = static_cast<recls_char_t>(toupper(drive));
				}
				else
				{
					drive = static_cast<recls_char_t>(tolower(drive));
				}
				cout << "  " << drive << endl;
				pathCheck += drive;
				pathCheck += ':';

				off += 2;
#elif defined(RECLS_PLATFORM_IS_WIN32)
                off += 2;
#else /* platform */
	            off += 0;
#endif /* RECLS_PLATFORM_API_WIN32 */
			}
            cout << "  " << setw(off + dir.length()) << dir << endl;

            typedef sequence_t::value_type::directory_parts_type  directory_parts_type;

#ifndef RECLS_PURE_API
            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                directory_parts_type    dirParts    =   fileEntry.get_directory_parts();
                int                     i;

                directory_parts_type::iterator  begin   =   dirParts.begin();
                directory_parts_type::iterator  end     =   dirParts.end();

                for(i = 0; begin != end; ++begin, ++i)
                {
                    string_t    dirPart =   *begin;

                    off += dirPart.length();
                    cout << "  " << setw(off) << dirPart << setw(1) << endl;

                    pathCheck += dirPart;
                    dirCheck += dirPart;
                }
            }
            else
#endif /* !RECLS_PURE_API */
            {
                off += dir.length();
                pathCheck += dir;
            }

            off += file.length();
            cout << "  " << setw(off) << file << setw(1) << endl;
            if(fileExt.length() > 0)
            {
                cout << "  " << setw(off - (1 + fileExt.length())) << fileName << setw(1) << endl;
                cout << "  " << setw(off) << fileExt << setw(1) << endl;
            }
            else
            {
                cout << "  " << setw(off) << fileName << setw(1) << endl;
            }

            fileCheck = fileName;
            if(0 < fileExt.length())
            {
                fileCheck += '.';
                fileCheck += fileExt;
            }

            pathCheck += file;

            // Now validate the components
            recls_assert(path == pathCheck);
            if(path != pathCheck)
            {
                cerr << "Path is different from path components" << endl 
					 << "\tpath:  \"" << path << "\"" << endl 
					 << "\tparts: \"" << pathCheck << "\"" << endl
					 << endl;

                abort();
            }
#ifdef RECLS_PLATFORM_API_WIN32
            recls_assert(NULL != host || (dirPath[0] == drive && dir == dirPath.c_str() + 2));
            recls_assert(NULL == host || dir == dirPath);
            if( (	NULL == host &&
					(	dirPath[0] != drive ||
						dir != dirPath.c_str() + 2)) ||
				(	NULL != host &&
					dir != dirPath))
#else
            recls_assert(fileEntry.is_UNC() || dirPath == dir);
            if(	!fileEntry.is_UNC() &&
				dirPath != dir)
#endif /* RECLS_PLATFORM_API_WIN32 */
            {
                cerr << "DirectoryPath is different from Directory" << endl 
					 << "\tdirPath: \"" << dirPath << "\"" << endl
					 << "\tdir:     \"" << dir << "\"" << endl
					 << endl;

                abort();
            }
            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                recls_assert(fileEntry.is_UNC() || dir == dirCheck);
                if( !fileEntry.is_UNC() &&
					dir != dirCheck)
                {
                    cerr << "Directory is different from directory components" << endl << "\tpath:  " << dir << "" << endl << "\tparts: " << dirCheck << endl << endl;

                    abort();
                }
            }
            recls_assert(file == fileCheck);
            if(file != fileCheck)
            {
                cerr << "File is different from file components" << endl << "\tpath:  " << "%s" << "" << endl << "\tparts: " << "%s" << endl << endl;

                abort();
            }

            if(fileEntry.is_readonly())
            {
                cout << "    - Read-only" << endl;
            }
            if(fileEntry.is_directory())
            {
                cout << "    - Directory" << endl;
            }
            else
            {
                cout << "    - File" << endl;
            }
            if(fileEntry.is_link())
            {
                cout << "    - Link" << endl;
            }
			if(fileEntry.is_UNC())
            {
                cout << "    - UNC" << endl;
            }
        }
    }

    return totalFound;
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
