/* /////////////////////////////////////////////////////////////////////////////
 * File:        nested_search_test.c
 *
 * Purpose:     Implementation file for the nested_search_test project.
 *
 * Created:     31st March 2004
 * Updated:     10th January 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2004-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>
#include <pthread.h>
#include <sched.h>
#include <stdlib.h>

#include <recls.h>

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit);

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

void *thread_proc(void *pfn)
{
	// TODO: Do some threading stuff
	hrecls_t	hSrch;
	recls_rc_t	rc	=	Recls_Search(NULL, Recls_GetWildcardsAll(), RECLS_F_RECURSIVE, &hSrch);

	for(; RECLS_SUCCEEDED(rc); rc = Recls_GetNext(hSrch))
	{
		recls_info_t	entry;

		rc = Recls_GetDetails(hSrch, &entry);

		if(RECLS_SUCCEEDED(rc))
		{
			printf("Outer: %.*s\n", entry->path.end - entry->path.begin, entry->path.begin);

			Recls_CloseDetails(entry);
		}
	}

	return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int         iRet    =   0;
    int         i;
	pthread_t	threads[2];

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
		}
	}

	/* . */

	for(i = 0; i < sizeof(threads) / sizeof(0[threads]); ++i)
	{
		void	*arg	=	NULL;

		if(0 != pthread_create(&threads[i], NULL, thread_proc, arg))
		{
			fprintf(stderr, "Failed to create thread %d\n", i);

			return 1;
		}
	}

	for(i = 0; i < sizeof(threads) / sizeof(0[threads]); ++i)
	{
		void	*arg	=	NULL;

		if(0 != pthread_join(threads[i], NULL))
		{
			fprintf(stderr, "Failed to join thread %d\n", i);

			return 1;
		}
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit)
{
    fprintf(stderr, "\n");
    fprintf(stderr, "  nested_search_test\n\n");

#if 0
    fprintf(stderr, "  USAGE: nested_search_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -w             - searches from the current working directory. The default\n");
    fprintf(stderr, "    -p<root-paths> - searches from the given root path(s), separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       -p\"c:\\windows;x:\\bin\"\n");
    fprintf(stderr, "    -r             - deletes readonly files\n");
    fprintf(stderr, "    -h             - searches from the roots of all drives on the system\n");
    fprintf(stderr, "    -d             - displays the path(s) searched\n");
    fprintf(stderr, "    -u             - do not act recursively\n");
    fprintf(stderr, "    -v             - verbose output. Prints time, attributes, size and path. (default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");
#endif /* 0 */

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
