/* /////////////////////////////////////////////////////////////////////////////
 * File:        speed_test.cpp
 *
 * Purpose:     Implementation file for the speed_test project.
 *
 * Created:     8th June 2004
 * Updated:     20th May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

#if defined(_MSC_VER)
# pragma warning(disable : 4786)
#endif /* _MSC_VER */

#if defined(__DMC__)
# include <iostream.h>
#else /* ? __DMC__ */
# include <iostream>

using std::cout;
using std::cerr;
using std::endl;
#endif /* __DMC__ */
#include <stdlib.h>

#include <stlsoft.h>
#if !defined(_STLSOFT_VER_1_8_3)
# error STLSoft version 1.8.3 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#if defined(__STLSOFT_COMPILER_IS_MSVC)
# pragma warning(disable : 4786)
#endif /* __STLSOFT_COMPILER_IS_MSVC */

#include <stlsoft/string_tokeniser.hpp>
#include <stlsoft/simple_string.hpp>

#include <winstl/performance_counter.hpp>

#include <reclspp.h>
#include <reclspp_filesearch.h>


/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason);

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int         iRet    =   0;
    int         i;
    char const	*pattern	=	NULL;
    char const	*rootDir	=	NULL;

	cout << "speed_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
			if(NULL == pattern)
			{
				pattern = arg;
			}
			else if(NULL == rootDir)
			{
				rootDir = arg;
			}
			else
			{
				usage(1, "Invalid argument(s) specified");
			}
		}
	}

	/* . */
	if( NULL == pattern ||
		NULL == rootDir)
	{
		usage(1, "Must specify pattern and rootDir");
	}

	winstl::performance_counter		counter;

	unsigned long					time_multi;
	unsigned long					time_recursive;

	for(int WARMUPS = 2; WARMUPS > 0; --WARMUPS)
	{
		// Recursive
		typedef stlsoft::basic_simple_string<char>			string_t;
		typedef stlsoft::string_tokeniser<string_t, char>	tokeniser_t;

		tokeniser_t					tokens(pattern, ';');
		tokeniser_t::const_iterator	b	=	tokens.begin();
		tokeniser_t::const_iterator	e	=	tokens.end();

		counter.start();
		for(; b != e; ++b)
		{
			reclspp::FileSearch	fs(rootDir, stlsoft::c_str_ptr(*b), recls::RECLS_F_FILES | recls ::RECLS_F_RECURSIVE);

			for(; fs.HasMoreElements(); )
			{
				fs.GetNext();
			}
		}
		counter.stop();
		time_recursive	=	(unsigned long)counter.get_milliseconds();

		// Multi-pattern
		counter.start();
		reclspp::FileSearch	fs(rootDir, pattern, recls::RECLS_F_FILES | recls ::RECLS_F_RECURSIVE);

		for(; fs.HasMoreElements(); )
		{
			fs.GetNext();
		}
		counter.stop();
		time_multi		=	(unsigned long)counter.get_milliseconds();
	}

	printf("Multi-pattern: %lu\n", time_multi);
	printf("Recursive:     %lu\n", time_recursive);

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason)
{
	cerr << endl;
    cerr << "  speed_test" << endl << endl;

	if(NULL != reason)
	{
		cerr << "  " << reason << endl << endl;
	}

#if 0
    cerr << "  speed_test" << endl << endl;
    cerr << "  USAGE: speed_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>" << endl;
    cerr << "    where:" << endl << endl;
    cerr << "    -w             - searches from the current working directory. The default" << endl;
    cerr << "    -p<root-paths> - searches from the given root path(s), separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       -p\"c:\\windows;x:\\bin\"" << endl;
    cerr << "    -r             - deletes readonly files" << endl;
    cerr << "    -h             - searches from the roots of all drives on the system" << endl;
    cerr << "    -d             - displays the path(s) searched" << endl;
    cerr << "    -u             - do not act recursively" << endl;
    cerr << "    -v             - verbose output. Prints time, attributes, size and path. (default)" << endl;
    cerr << "    -s             - succinct output. Prints path only" << endl;
    cerr << "    <search-spec>  - one or more file search specifications, separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       \"*.exe\"" << endl;
    cerr << "                       \"myfile.ext\"" << endl;
    cerr << "                       \"*.exe;*.dll\"" << endl;
    cerr << "                       \"*.xl?;report.*\"" << endl;
	cerr << endl;
	cerr << "  Contact " << _nameSynesisSoftware << endl;
	cerr << "    at \"" << _wwwSynesisSoftware_SystemTools << "\"," << endl;
	cerr << "    or, via email, at \"" << _emailSynesisSoftware_SystemTools << "\"" << endl;
	cerr << endl;
#endif /* 0 */

    if(bExit)
    {
        exit(1);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
